#!/usr/bin/perl -w

# By Dan Martin, 2010 
# 360 Analytics Limited, http://www.360analytics.co.uk,  dan at 360 analytics dot co dot uk
# This script is licensed under the GPLv3 license.
# For info please contact us via http://www.360analytics.co.uk/contact/
# v1.1

use strict;
use Data::Dumper;
use Getopt::Long;

require 'shellwords.pl';

my %acls;
my %aclints;
my %unusedints;

my $logfile = $ARGV[0];
my $aclname = $ARGV[1];

open(LOGFILE1, "<$logfile") or die "EEEEEeeeeeeee Can't open $logfile: $!\n";;

if ($aclname =~ /^ACLNAME$/) {
	my ($acl, $aclint) ;
	open(ACLFILE1, "<$aclname") or die "EEEEEeeeeeeee Can't open $aclname: $!\n";

	while (<ACLFILE1>) {
		chomp;
		my @aclline = shellwords($_);
		$acl = $aclline[1] ;
		$aclint  = $aclline[4] ;
		$aclints{$aclint} = $acl;
	}
}

print "\n!!WOOT!! Cisco ACL's built while bollock juggling!!

...................................
Mode or ACL Name: $aclname
Search Strings Used:\n";
if (defined ($ARGV[2])) { print "	$ARGV[2]\n"};
if (defined ($ARGV[3])) { print "	$ARGV[3]\n"};

print "\nACL to interface groupings in use
...................................\n";
#print "\n\nACLs\n#################################################################################\n\n". Dumper(%aclints);

foreach my $chosenacls (sort keys %aclints) {
	print "Interface $chosenacls is associated with $aclints{$chosenacls}\n";
}

print "\n\nIf your syslogger has added its own header to the beginning of each line\n";
print "cut and paste one header below, if not just hit return
...................................\n";

my $offset;
my $linehead = <STDIN>;
my @linehead = shellwords($linehead);
if ($#linehead >= 1) {
	$offset  	= $#linehead + 1;
} else {
	$offset 	= 0 ;
}

print "\n\nPulling Logs
...................................\n";

my $count;	
while (<LOGFILE1>) {
	$count++;
        chomp;

	if (defined ($ARGV[2])) {
        	next unless /$ARGV[2]/ ;

		if (defined ($ARGV[3])) { 
        		next unless /$ARGV[3]/ ;
		}
	}

        if (/\%ASA-6-302013/) {

                my ($source, $sourceintip, $sourceint, $sourceip, $sourceport);
                my ($dest, $destintip, $destint, $destip, $destport);

                my @line = shellwords($_);
                ($source, $dest) = ($line[11 + $offset], $line[14 + $offset]) ;

                ($sourceintip, $sourceport) 	= split(/\//, $source);
                ($destintip, $destport) 	= split(/\//, $dest);

                ($destint, $destip) 		= split(/\:/, $destintip);
                ($sourceint, $sourceip)		= split(/\:/, $sourceintip);

		if ($aclname =~ /^DEBUG$/) {
                	print "TCP SrcInt: $sourceint	SrcIP: $sourceip	DstInt: $destint	DestIP: $destip	DestPort: $destport\n";

		} elsif ($aclname =~ /^SRCINT$/) {
			$acls{"access-list $sourceint extended permit tcp host $sourceip host $destip eq $destport"}++;

		} elsif ($aclname =~ /^ACLNAME$/) {
			if (defined ( $aclints{$sourceint} )) {
				$acls{"access-list $aclints{$sourceint} extended permit tcp host $sourceip host $destip eq $destport"}++;
			} else {
				$unusedints{$sourceint}++;
			}
		} else {
			$acls{"access-list $aclname extended permit tcp host $sourceip host $destip eq $destport"}++;
		}

        } elsif (/\%ASA-6-302015/) {

                my ($source, $sourceintip, $sourceint, $sourceip, $sourceport);
                my ($dest, $destintip, $destint, $destip, $destport);

                my @line = shellwords($_);
                ($source, $dest) = ($line[11 + $offset], $line[14 + $offset]) ;

                ($sourceintip, $sourceport) 	= split(/\//, $source);
                ($destintip, $destport) 	= split(/\//, $dest);

                ($destint, $destip) 		= split(/\:/, $destintip);
                ($sourceint, $sourceip)		= split(/\:/, $sourceintip);

		if ($aclname =~ /^DEBUG$/) {
                	print "UDP SrcInt: $sourceint	SrcIP: $sourceip	DstInt: $destint	DestIP: $destip	DestPort: $destport\n";

		} elsif ($aclname =~ /^SRCINT$/) {
			$acls{"access-list $sourceint extended permit udp host $sourceip host $destip eq $destport"}++;

		} elsif ($aclname =~ /^ACLNAME$/) {
			if (defined ( $aclints{$sourceint} )) {
				$acls{"access-list $aclints{$sourceint} extended permit udp host $sourceip host $destip eq $destport"}++;
			} else {
				$unusedints{$sourceint}++;
			}
		} else {
			$acls{"access-list $aclname extended permit udp host $sourceip host $destip eq $destport"}++;
		}
	}
}
print "$count log entries processed\n\n";

#print "\n\nACLs\n#################################################################################\n\n". Dumper(%acls);

print "\nInterfaces not considered on which traffic matched
...................................\n";
#print "\n\nINTERFACESs\n#################################################################################\n\n". Dumper(%unusedints);

foreach my $droppedints (sort keys %unusedints) {
	print "Interface $droppedints had $unusedints{$droppedints} hits\n";
}

print "\n\nACLs to permit the specified traffic from the logs:
........................................................\n";

foreach my $acl (sort keys(%acls)) {
	print "$acl\n";
}

print "\n\n";

